import { User, UserWithKudosu, UserWithCountryCoverGroupsStatisticsrulesets, UserWithCountryCoverGroupsStatisticsSupport, UserExtended, UserExtendedWithStatisticsrulesets, KudosuHistory } from "./user.js";
import { Beatmap, BeatmapExtendedWithFailtimesBeatmapsetextended, BeatmapExtended, BeatmapPlaycount, BeatmapDifficultyAttributes, BeatmapDifficultyAttributesOsu, BeatmapDifficultyAttributesTaiko, BeatmapDifficultyAttributesFruits, BeatmapDifficultyAttributesMania, Beatmapset, BeatmapsetExtendedWithBeatmapExtended, BeatmapsetExtendedPlus, BeatmapPack } from "./beatmap.js";
import { Room, Leader, PlaylistItem, MultiplayerScores, Match, MatchInfo } from "./multiplayer.js";
import { Score, ScoreWithUser, ScoreWithUserBeatmapBeatmapset, BeatmapUserScore } from "./score.js";
import { Rankings, RankingsCountry, Spotlight, RankingsSpotlight } from "./ranking.js";
import { EventAchievement, EventBeatmapsetApprove, EventBeatmapsetRevive, EventBeatmapsetUpdate, EventBeatmapsetUpload, EventRank, EventRankLost, EventUserSupportAgain, EventUserSupportFirst, EventUserSupportGift, EventUsernameChange } from "./event.js";
import { ChangelogBuildWithUpdatestreamsChangelogentries, ChangelogBuildWithChangelogentriesVersions, UpdateStream } from "./changelog.js";
import { WikiPage } from "./wiki.js";
import { SearchResultUser, SearchResultWiki } from "./home.js";
import { Rulesets, Mod, Scope } from "./misc.js";
export { User, UserWithKudosu, UserWithCountry, UserWithCountryCover, UserWithCountryCoverGroupsStatisticsrulesets, UserWithCountryCoverGroupsStatisticsSupport, UserExtended, UserExtendedWithStatisticsrulesets, UserStatistics, UserStatisticsWithUser, UserStatisticsWithCountryrank, KudosuHistory, ProfileBanner } from "./user.js";
export { Beatmap, BeatmapExtendedWithFailtimesBeatmapsetextended, BeatmapWithBeatmapset, BeatmapWithBeatmapsetChecksumMaxcombo, BeatmapExtended, BeatmapPlaycount, BeatmapDifficultyAttributes, BeatmapDifficultyAttributesOsu, BeatmapDifficultyAttributesTaiko, BeatmapDifficultyAttributesFruits, BeatmapDifficultyAttributesMania, Beatmapset, BeatmapsetExtended, BeatmapExtendedWithFailtimes, BeatmapsetExtendedWithBeatmapExtended, BeatmapsetExtendedPlus, BeatmapPack, RankStatus } from "./beatmap.js";
export { Room, Leader, PlaylistItem, MultiplayerScore, MultiplayerScores, Match, MatchInfo } from "./multiplayer.js";
export { Score, ScoreWithMatch, ScoreWithUser, ScoreWithUserBeatmap, ScoreWithUserBeatmapBeatmapset, BeatmapUserScore } from "./score.js";
export { Rankings, RankingsCountry, Spotlight, SpotlightWithParticipantcount, RankingsSpotlight } from "./ranking.js";
export { Event, EventUser, EventBeatmap, EventBeatmapset, EventBeatmapPlaycount, EventBeatmapsetDelete, EventAchievement, EventBeatmapsetApprove, EventBeatmapsetRevive, EventBeatmapsetUpdate, EventBeatmapsetUpload, EventRank, EventRankLost, EventUserSupportAgain, EventUserSupportFirst, EventUserSupportGift, EventUsernameChange } from "./event.js";
export { ChangelogBuildWithUpdatestreams, ChangelogBuildWithUpdatestreamsChangelogentries, ChangelogBuildWithChangelogentriesVersions, UpdateStream } from "./changelog.js";
export { WikiPage } from "./wiki.js";
export { SearchResultUser, SearchResultWiki } from "./home.js";
export { Rulesets, Mod, Scope } from "./misc.js";
/**
 * Generates a link for users to click on in order to use your application!
 * @param client_id The Client ID, find it at https://osu.ppy.sh/home/account/edit#new-oauth-application
 * @param redirect_uri The specified Application Callback URL, aka where the user will be redirected upon clicking the button to authorize
 * @param scopes What you want to do with/as the user
 * @returns The link people should click on
 */
export declare function generateAuthorizationURL(client_id: number, redirect_uri: string, scopes: Scope[]): string;
/**
 * If the `API` throws an error, it should always be an `APIError`!
 */
export declare class APIError {
    message: string;
    server: string;
    endpoint: string;
    parameters: string;
    /**
     * @param message The reason why things didn't go as expected
     * @param server The server to which the request was sent
     * @param endpoint The type of resource that was requested from the server
     * @param parameters The filters that were used to specify what resource was wanted
     */
    constructor(message: string, server: string, endpoint: string, parameters: string);
}
/**
 * You can create an API instance using its `createAsync` function!
 */
export declare class API {
    client: {
        id: number;
        secret: string;
    };
    /**
     * Should always be "Bearer"
     */
    token_type: string;
    expires: Date;
    access_token: string;
    /**
     * Valid for an unknown amount of time, allows you to get a new token without going through the Authorization Code Grant!
     * Use the API's `refreshToken` function to do that
     */
    refresh_token?: string;
    /**
     * The osu! user id of the user who went through the Authorization Code Grant
     */
    user?: number;
    scopes: Scope[];
    /**
     * (default `none`) Which events should be logged
     */
    verbose: "none" | "errors" | "all";
    /**
     * (default `https://osu.ppy.sh`) The base url of the server where the requests should land
     * @remarks For tokens, requests will be sent to the `oauth/token` route, other requests will be sent to the `api/v2` route
     */
    server: string;
    /**
     * Use the API's `createAsync` instead of the default constructor if you don't have at least an access_token!
     * `createAsync` should always be your way of creating API instances!!
     */
    constructor(client?: {
        id: number;
        secret: string;
    }, token_type?: string, expires?: Date, access_token?: string, scopes?: Scope[], refresh_token?: string, user?: number, verbose?: "none" | "errors" | "all", server?: string);
    /**
     * Use this instead of `console.log` to log any information
     * @param is_error Is the logging happening because of an error?
     * @param to_log Whatever you would put between the parentheses of `console.log()`
     */
    private log;
    private obtainToken;
    /**
     * The normal way to create an API instance! Make sure to `await` it
     * @param client The ID and the secret of your client, can be found on https://osu.ppy.sh/home/account/edit#new-oauth-application
     * @param user If the instance is supposed to represent a user, use their Authorization Code and the Application Callback URL of your application!
     * @returns A promise with an API instance
     */
    static createAsync(client: {
        id: number;
        secret: string;
    }, user?: {
        code: string;
        redirect_uri: string;
    }, verbose?: "none" | "errors" | "all", server?: string): Promise<API>;
    refreshToken(): Promise<boolean>;
    /**
     * @param endpoint What comes in the URL after `api/`
     * @param parameters The things to specify in the request, such as the beatmap_id when looking for a beatmap
     * @param number_try How many attempts there's been to get the data
     * @returns A Promise with either the API's response or `false` upon failing
     */
    private request;
    /**
     * Get extensive user data about the authorized user
     * @param ruleset Defaults to the user's default/favourite Ruleset
     * @scope identify
     */
    getResourceOwner(ruleset?: Rulesets): Promise<UserExtendedWithStatisticsrulesets>;
    /**
     * Get extensive user data about whoever you want!
     * @param user An object with either the id or the username of the user you're trying to get
     * @param ruleset Defaults to the user's default/favourite Ruleset
     */
    getUser(user: {
        id?: number;
        username?: string;
    } | User, ruleset?: Rulesets): Promise<UserExtended>;
    /**
     * Get user data for up to 50 users at once!
     * @param ids An array composed of the ids of the users you want
     */
    getUsers(ids: number[]): Promise<UserWithCountryCoverGroupsStatisticsrulesets[]>;
    /**
     * Get "notable" scores from a user
     * @param user The user who set the scores
     * @param type Do you want scores: in the user's top 100, that are top 1 on a beatmap, that have been recently set?
     * @param limit The maximum amount of scores to be returned
     * @param ruleset The Ruleset the scores were made in, defaults to the user's default/favourite Ruleset
     * @param include_fails (defaults to false) Do you want scores where the user didn't survive or quit the map?
     * @param offset How many elements that would be at the top of the returned array get skipped (while still filling the array up to the limit)
     */
    getUserScores(user: {
        id: number;
    } | User, type: "best" | "firsts" | "recent", limit?: number, ruleset?: Rulesets, include_fails?: boolean, offset?: number): Promise<ScoreWithUserBeatmapBeatmapset[]>;
    /**
     * Get beatmaps favourited or made by a user!
     * @param user The user in question
     * @param type The relation between the user and the beatmaps
     * @param limit (defaults to 5) The maximum amount of elements returned in the array
     * @param offset How many elements that would be at the top of the returned array get skipped (while still filling the array up to the limit)
     */
    getUserBeatmaps(user: {
        id: number;
    } | User, type: "favourite" | "graveyard" | "guest" | "loved" | "nominated" | "pending" | "ranked", limit?: number, offset?: number): Promise<BeatmapsetExtendedWithBeatmapExtended[]>;
    /**
     * Get the beatmaps most played by a user!
     * @param user The user who played the beatmaps
     * @param limit (defaults to 5) The maximum amount of elements returned in the array
     * @param offset How many elements that would be at the top of the returned array get skipped (while still filling the array up to the limit)
     */
    getUserMostPlayed(user: {
        id: number;
    } | User, limit?: number, offset?: number): Promise<BeatmapPlaycount[]>;
    /**
     * Get an array of Events of different `type`s that relate to a user's activity during the last 31 days! (or 100 activities, whatever comes first)
     * @param user The user in question
     * @param limit (defaults to 5) The maximum amount of elements returned in the array
     * @param offset How many elements that would be at the top of the returned array get skipped (while still filling the array up to the limit)
     */
    getUserRecentActivity(user: {
        id: number;
    } | User, limit?: number, offset?: number): Promise<Array<EventAchievement | EventBeatmapsetApprove | EventBeatmapsetRevive | EventBeatmapsetUpdate | EventBeatmapsetUpload | EventRank | EventRankLost | EventUserSupportAgain | EventUserSupportFirst | EventUserSupportGift | EventUsernameChange>>;
    /**
     * Get data about the activity of a user kudosu-wise!
     * @param user The user in question
     * @param limit (defaults to 5) The maximum amount of activities in the returned array
     * @param offset How many elements that would be at the top of the returned array get skipped (while still filling the array up to the limit)
     */
    getUserKudosu(user: {
        id: number;
    } | User, limit?: number, offset?: number): Promise<KudosuHistory[]>;
    /**
     * Get user data of each friend of the authorized user
     * @scope friends.read
     */
    getFriends(): Promise<UserWithCountryCoverGroupsStatisticsSupport[]>;
    /**
     * Get extensive beatmap data about whichever beatmap you want!
     * @param beatmap An object with the id of the beatmap you're trying to get
     */
    getBeatmap(beatmap: {
        id: number;
    } | Beatmap): Promise<BeatmapExtendedWithFailtimesBeatmapsetextended>;
    /**
     * Get extensive beatmap data for up to 50 beatmaps at once!
     * @param ids An array composed of the ids of the beatmaps you want
     */
    getBeatmaps(ids?: number[]): Promise<BeatmapExtended[]>;
    /**
     * Get various data about the difficulty of a beatmap!
     * @remarks You may want to use getBeatmapDifficultyAttributesOsu (or Taiko or whatever) instead for better type safety
     * @param beatmap The Beatmap in question
     * @param mods (defaults to No Mod) (will ignore mod settings) Can be a bitset of mods, an array of mod acronyms ("DT" for DoubleTime), or an array of Mods
     * @param ruleset (defaults to the ruleset the beatmap was intended for) Useful to specify if the beatmap is a convert
     */
    getBeatmapDifficultyAttributes(beatmap: {
        id: number;
    } | Beatmap, mods?: Mod[] | string[] | number, ruleset?: Rulesets): Promise<BeatmapDifficultyAttributes | BeatmapDifficultyAttributesOsu | BeatmapDifficultyAttributesTaiko | BeatmapDifficultyAttributesFruits | BeatmapDifficultyAttributesMania>;
    /**
     * Get various data about the difficulty of an osu! beatmap!
     * @param beatmap The Beatmap in question
     * @param mods (defaults to No Mod) (will ignore mod settings) Can be a bitset of mods, an array of mod acronyms ("DT" for DoubleTime), or an array of Mods
     */
    getBeatmapDifficultyAttributesOsu(beatmap: {
        id: number;
    } | Beatmap, mods?: Mod[] | string[] | number): Promise<BeatmapDifficultyAttributesOsu>;
    /**
     * Get various data about the difficulty of a taiko beatmap!
     * @param beatmap The Beatmap in question
     * @param mods (defaults to No Mod) (will ignore mod settings) Can be a bitset of mods, an array of mod acronyms ("DT" for DoubleTime), or an array of Mods
     */
    getBeatmapDifficultyAttributesTaiko(beatmap: {
        id: number;
    } | Beatmap, mods?: Mod[] | string[] | number): Promise<BeatmapDifficultyAttributesTaiko>;
    /**
     * Get various data about the difficulty of a ctb beatmap!
     * @param beatmap The Beatmap in question
     * @param mods (defaults to No Mod) (will ignore mod settings) Can be a bitset of mods, an array of mod acronyms ("DT" for DoubleTime), or an array of Mods
     */
    getBeatmapDifficultyAttributesFruits(beatmap: {
        id: number;
    } | Beatmap, mods?: Mod[] | string[] | number): Promise<BeatmapDifficultyAttributesFruits>;
    /**
     * Get various data about the difficulty of a mania beatmap!
     * @param beatmap The Beatmap in question
     * @param mods (defaults to No Mod) (will ignore mod settings) Can be a bitset of mods, an array of mod acronyms ("DT" for DoubleTime), or an array of Mods
     */
    getBeatmapDifficultyAttributesMania(beatmap: {
        id: number;
    } | Beatmap, mods?: Mod[] | string[] | number): Promise<BeatmapDifficultyAttributesMania>;
    /**
     * Get the top scores of a beatmap!
     * @param beatmap The Beatmap in question
     * @param ruleset The Ruleset used to make the scores, useful if they were made on a convert
     * @param mods (2023-11-16) Currently doesn't do anything
     * @param type (2023-11-16) Currently doesn't do anything
     */
    getBeatmapScores(beatmap: {
        id: number;
    } | Beatmap, ruleset?: Rulesets, mods?: string[], type?: string): Promise<ScoreWithUser[]>;
    /**
     * Get the score on a beatmap made by a specific user (with specific mods and on a specific ruleset if needed)
     * @param beatmap The Beatmap the score was made on
     * @param user The User who made the score
     * @param mods The Mods used to make the score, defaults to any, you can use `["NM"]` to filter out scores with mods
     * @param ruleset The Ruleset used to make the score, useful if it was made on a convert
     * @returns An Object with the position of the score according to the specified Mods and Ruleset, and with the score itself
     */
    getBeatmapUserScore(beatmap: {
        id: number;
    } | Beatmap, user: {
        id: number;
    } | User, mods?: string[], ruleset?: Rulesets): Promise<BeatmapUserScore>;
    /**
     * Get the score on a beatmap made by a specific user (with the possibility to specify if the scores are on a convert)
     * @param beatmap The Beatmap the scores were made on
     * @param user The User who made the scores
     * @param ruleset The Ruleset used to make the scores, defaults to the Ruleset the Beatmap was made for
     */
    getBeatmapUserScores(beatmap: {
        id: number;
    } | Beatmap, user: {
        id: number;
    } | User, ruleset?: Rulesets): Promise<Score[]>;
    /**
     * Get extensive beatmapset data about whichever beatmapset you want!
     * @param beatmapset An object with the id of the beatmapset you're trying to get
     */
    getBeatmapset(beatmapset: {
        id: number;
    } | Beatmapset): Promise<BeatmapsetExtendedPlus>;
    /**
     * Get data about a BeatmapPack using its tag!
     * @param pack An object with the tag of the beatmappack you're trying to get
     * @remarks Currently in https://osu.ppy.sh/beatmaps/packs, when hovering a pack, its link with its tag should show up on your browser's bottom left
     */
    getBeatmapPack(pack: {
        tag: string;
    } | BeatmapPack): Promise<BeatmapPack>;
    /**
     * Get an Array of up to 100 BeatmapPacks of a specific type!
     * @param type The type of the BeatmapPacks, defaults to "standard"
     */
    getBeatmapPacks(type?: "standard" | "featured" | "tournament" | "loved" | "chart" | "theme" | "artist"): Promise<BeatmapPack[]>;
    /**
     * Get details about the version/update/build of something related to osu!
     * @param stream The name of the thing related to osu!, like `lazer`, `web`, `cuttingedge`, `beta40`, `stable40`
     * @param build The name of the version! Usually something like `2023.1026.0` for lazer, or `20230326` for stable
     */
    getChangelogBuild(stream: string, build: string): Promise<ChangelogBuildWithChangelogentriesVersions>;
    /**
     * Get up to 21 versions/updates/builds!
     * @param versions Get builds that were released before/after (and including) those versions (use the name of the versions, e.g. `2023.1109.0`)
     * @param max_id Filter out builds that have an id higher than this (this takes priority over `versions.to`)
     * @param stream Only get builds from a specific stream
     * @param message_formats Each element of `changelog_entries` will have a `message` property if `markdown`, `message_html` if `html`, defaults to both
     */
    getChangelogBuilds(versions?: {
        from?: string;
        to?: string;
    }, max_id?: number, stream?: string, message_formats?: ("html" | "markdown")[]): Promise<ChangelogBuildWithUpdatestreamsChangelogentries[]>;
    /**
     * An effective way to get all available streams, as well as their latest version!
     * @example ```ts
     * const names_of_streams = (await api.getChangelogStreams()).map(s => s.name)
     * ```
     */
    getChangelogStreams(): Promise<UpdateStream[]>;
    /**
     * Get data about a lazer multiplayer room (realtime or playlists)!
     * @param room An object with the id of the room, is at the end of its URL (after `/multiplayer/rooms/`)
     */
    getRoom(room: {
        id: number;
    } | Room): Promise<Room>;
    /**
     * Get rooms that are active, that have ended, that the user participated in, that the user made, or just simply any room!
     * @param mode Self-explanatory enough, defaults to `active`
     * @scope public
     */
    getRooms(mode?: "active" | "all" | "ended" | "participated" | "owned"): Promise<Room[]>;
    /**
     * Get the room stats of all the users of that room!
     * @param room An object with the id of the room in question
     * @scope public
     */
    getRoomLeaderboard(room: {
        id: number;
    } | Room): Promise<Leader[]>;
    /**
     * Get the scores on a specific item of a room!
     * @param item An object with the id of the item in question, as well as the id of the room
     * @param limit How many scores maximum? Defaults to 50, the maximum the API will return
     * @param sort Sort by scores, ascending or descending? Defaults to descending
     * @param cursor_string Use a MultiplayerScores' `params` and `cursor_string` to get the next page (scores 51 to 100 for example)
     * @remarks (2023-11-10) Items are broken for multiplayer (real-time) rooms, not for playlists (like spotlights), that's an osu! bug
     * https://github.com/ppy/osu-web/issues/10725
     */
    getPlaylistItemScores(item: {
        id: number;
        room_id: number;
    } | PlaylistItem, limit?: number, sort?: "score_asc" | "score_desc", cursor_string?: string): Promise<MultiplayerScores>;
    /**
     * Get data of a multiplayer lobby from the stable (non-lazer) client that have URLs with `community/matches` or `mp`
     * @param id Can be found at the end of the URL of said match
     */
    getMatch(id: number): Promise<Match>;
    /**
     * Gets the info of the 50 most recently created stable (non-lazer) matches, descending order (most recent is at index 0)
     */
    getMatches(): Promise<MatchInfo[]>;
    /**
     * Get the top 50 players who have the most total kudosu!
     */
    getKudosuRanking(): Promise<UserWithKudosu[]>;
    /**
     * Get the top players of the game, with some filters!
     * @param ruleset Self-explanatory, is also known as "Gamemode"
     * @param type Rank players by their performance points or by their ranked score?
     * @param page (defaults to 1) Imagine `Rankings` as a page, it can only have a maximum of 50 players, while 50 others may be on the next one
     * @param filter What kind of players do you want to see? Defaults to `all`, `friends` has no effect if no authorized user
     * @param country Only get players from a specific country, using its ISO 3166-1 alpha-2 country code! (France would be `FR`, United States `US`)
     * @param variant If `type` is `performance` and `ruleset` is mania, choose between 4k and 7k!
     */
    getRanking(ruleset: Rulesets, type: "performance" | "score", page?: number, filter?: "all" | "friends", country?: string, variant?: "4k" | "7k"): Promise<Rankings>;
    /**
     * Get the top countries of a specific ruleset!
     * @param ruleset On which Ruleset should the countries be compared?
     * @param page (defaults to 1) Imagine `Rankings` as a page, it can only have a maximum of 50 countries, while 50 others may be on the next one
     */
    getCountryRanking(ruleset: Rulesets, page?: number): Promise<RankingsCountry>;
    /**
     * Get the rankings of a spotlight from 2009 to 2020 on a specific ruleset!
     * @param ruleset Each spotlight has a different ranking (and often maps) depending on the ruleset
     * @param spotlight The spotlight in question
     * @param filter What kind of players do you want to see? Defaults to `all`, `friends` has no effect if no authorized user
     */
    getSpotlightRanking(ruleset: Rulesets, spotlight: {
        id: number;
    } | Spotlight, filter?: "all" | "friends"): Promise<RankingsSpotlight>;
    /**
     * Get ALL legacy spotlights! (2009-2020, somewhat known as charts/ranking charts, available @ https://osu.ppy.sh/rankings/osu/charts)
     * @remarks The data for newer spotlights (2020-, somewhat known as seasons) can be obtained through `getRoom()`
     * but you can't really get their id without going through the website's URLs (https://osu.ppy.sh/seasons/latest) as far as I know :(
     */
    getSpotlights(): Promise<Spotlight[]>;
    /**
     * Look for a user like you would on the website!
     * @param query What you would put in the searchbar
     * @param page (defaults to 1) You normally get the first 20 results, but if page is 2, you'd get results 21 to 40 instead for example!
     */
    searchUser(query: string, page?: number): Promise<SearchResultUser>;
    /**
     * Look for a wiki page like you would on the website!
     * @param query What you would put in the searchbar
     * @param page (defaults to 1) You normally get the first 50 results, but if page is 2, you'd get results 51 to 100 instead for example!
     */
    searchWiki(query: string, page?: number): Promise<SearchResultWiki>;
    /**
     * Get a wiki page!
     * @param path What's in the page's URL after `https://osu.ppy.sh/wiki/` (so the title, after the subtitle if there is a subtitle)
     * (An example for `https://osu.ppy.sh/wiki/en/Game_mode/osu!` would be `Game_mode/osu!`)
     * @param locale (defaults to "en") The BCP 47 language (sub)tag, lowercase (for example, for the article in french, use "fr")
     */
    getWikiPage(path: string, locale?: string): Promise<WikiPage>;
}
